@@ -1,29 +1,35 @@
 import numpy as np
-from gym import utils
-from gym.envs.mujoco import mujoco_env
+from gymnasium import utils
+from gymnasium.envs.mujoco import mujoco_env
+from gymnasium.spaces import Box
 
 
-def mass_center(model, sim):
+def mass_center(model, data):
     mass = np.expand_dims(model.body_mass, 1)
-    xpos = sim.data.xipos
+    xpos = data.xipos
     return (np.sum(mass * xpos, 0) / np.sum(mass))[0]
 
 
 # Obtained from https://raw.githubusercontent.com/JannerM/mbpo/master/mbpo/env/humanoid.py
 class HumanoidTruncatedObsEnv(mujoco_env.MujocoEnv, utils.EzPickle):
-    """
-    COM inertia (cinert), COM velocity (cvel), actuator forces (qfrc_actuator),
-    and external forces (cfrc_ext) are removed from the observation.
-    Otherwise identical to Humanoid-v2 from
-    https://github.com/openai/gym/blob/master/gym/envs/mujoco/humanoid.py
-    """
+    metadata = {
+        "render_modes": [
+            "human",
+            "rgb_array",
+            "depth_array",
+        ],
+        "render_fps": 67,
+    }
 
-    def __init__(self):
-        mujoco_env.MujocoEnv.__init__(self, "humanoid.xml", 5)
+    def __init__(self, render_mode: str = None):
+        observation_space = Box(low=-np.inf, high=np.inf, shape=(45,), dtype=np.float64)
+        mujoco_env.MujocoEnv.__init__(
+            self, "humanoid.xml", 5, observation_space, render_mode=render_mode
+        )
         utils.EzPickle.__init__(self)
 
     def _get_obs(self):
-        data = self.sim.data
+        data = self.data
         return np.concatenate(
             [
                 data.qpos.flat[2:],
@@ -36,22 +42,27 @@ class HumanoidTruncatedObsEnv(mujoco_env.MujocoEnv, utils.EzPickle):
         )
 
     def step(self, a):
-        pos_before = mass_center(self.model, self.sim)
+        pos_before = mass_center(self.model, self.data)
         self.do_simulation(a, self.frame_skip)
-        pos_after = mass_center(self.model, self.sim)
+        pos_after = mass_center(self.model, self.data)
         alive_bonus = 5.0
-        data = self.sim.data
+        data = self.data
         lin_vel_cost = 0.25 * (pos_after - pos_before) / self.model.opt.timestep
         quad_ctrl_cost = 0.1 * np.square(data.ctrl).sum()
         quad_impact_cost = 0.5e-6 * np.square(data.cfrc_ext).sum()
         quad_impact_cost = min(quad_impact_cost, 10)
         reward = lin_vel_cost - quad_ctrl_cost - quad_impact_cost + alive_bonus
-        qpos = self.sim.data.qpos
-        done = bool((qpos[2] < 1.0) or (qpos[2] > 2.0))
+        qpos = self.data.qpos
+        terminated = bool((qpos[2] < 1.0) or (qpos[2] > 2.0))
+
+        if self.render_mode == "human":
+            self.render()
+
         return (
             self._get_obs(),
             reward,
-            done,
+            terminated,
+            False,
             dict(
                 reward_linvel=lin_vel_cost,
                 reward_quadctrl=-quad_ctrl_cost,
