@@ -4,8 +4,7 @@
 # LICENSE file in the root directory of this source tree.
 import abc
 import pathlib
-import warnings
-from typing import Any, Dict, Optional, Sequence, Tuple, Union, cast
+from typing import Any, Dict, Optional, Sequence, Tuple, Union
 
 import torch
 from torch import nn as nn
@@ -13,17 +12,6 @@ from torch import nn as nn
 from mbrl.models.util import to_tensor
 from mbrl.types import ModelInput, TransitionBatch
 
-# TODO: these are temporary, eventually it will be tuple(tensor, dict), keeping this
-#  for back-compatibility with v0.1.x, and will be removed in v0.2.0
-LossOutput = Union[torch.Tensor, Tuple[torch.Tensor, Dict[str, Any]]]
-UpdateOutput = Union[float, Tuple[float, Dict[str, Any]]]
-
-
-_NO_META_WARNING_MSG = (
-    "Starting in version v0.2.0, `model.loss()`, model.update(), and model.eval_score() "
-    "must all return a tuple with (loss, metadata)."
-)
-
 
 # ---------------------------------------------------------------------------
 #                           ABSTRACT MODEL CLASS
@@ -48,6 +36,8 @@ class Model(nn.Module, abc.ABC):
             of this.
     """
 
+    _MODEL_FNAME = "model.pth"
+
     def __init__(
         self,
         device,
@@ -57,23 +47,31 @@ class Model(nn.Module, abc.ABC):
         super().__init__()
         self.device = device
 
-    def _process_batch(self, batch: TransitionBatch) -> Tuple[torch.Tensor, ...]:
+    def _process_batch(
+        self, batch: TransitionBatch, as_float: bool = True
+    ) -> Tuple[torch.Tensor, ...]:
         def _convert(x):
-            return to_tensor(x).to(self.device)
+            if x is None:
+                return None
+            res = to_tensor(x).to(self.device)
+            if as_float:
+                return res.float()
+            return res
 
         return (
             _convert(batch.obs),
             _convert(batch.act),
             _convert(batch.next_obs),
-            _convert(batch.rewards).view(-1, 1),
-            _convert(batch.dones).view(-1, 1),
+            None if batch.rewards is None else _convert(batch.rewards),
+            None if batch.terminateds is None else _convert(batch.terminateds),
+            None if batch.truncateds is None else _convert(batch.truncateds),
         )
 
-    def forward(self, x: ModelInput, **kwargs) -> Tuple[torch.Tensor, ...]:
+    def forward(self, x: torch.Tensor, *args, **kwargs) -> Tuple[torch.Tensor, ...]:
         """Computes the output of the dynamics model.
 
         Args:
-            x (tensor or batch of transitions): the input to the model.
+            x (tensor): the input to the model.
 
         Returns:
             (tuple of tensors): all tensors predicted by the model (e.g., .mean and logvar).
@@ -85,7 +83,7 @@ class Model(nn.Module, abc.ABC):
         self,
         model_in: ModelInput,
         target: Optional[torch.Tensor] = None,
-    ) -> LossOutput:
+    ) -> Tuple[torch.Tensor, Dict[str, Any]]:
         """Computes a loss that can be used to update the model using backpropagation.
 
         Args:
@@ -103,7 +101,7 @@ class Model(nn.Module, abc.ABC):
     @abc.abstractmethod
     def eval_score(
         self, model_in: ModelInput, target: Optional[torch.Tensor] = None
-    ) -> LossOutput:
+    ) -> Tuple[torch.Tensor, Dict[str, Any]]:
         """Computes an evaluation score for the model over the given input/target.
 
         This method should compute a non-reduced score for the model, intended mostly for
@@ -133,7 +131,7 @@ class Model(nn.Module, abc.ABC):
         model_in: ModelInput,
         optimizer: torch.optim.Optimizer,
         target: Optional[torch.Tensor] = None,
-    ) -> UpdateOutput:
+    ) -> Tuple[float, Dict[str, Any]]:
         """Updates the model using backpropagation with given input and target tensors.
 
         Provides a basic update function, following the steps below:
@@ -157,29 +155,16 @@ class Model(nn.Module, abc.ABC):
         """
         self.train()
         optimizer.zero_grad()
-        loss_and_maybe_meta = self.loss(model_in, target)
-        if isinstance(loss_and_maybe_meta, tuple):
-            # TODO - v0.2.0 remove this back-compatibility logic
-            loss = cast(torch.Tensor, loss_and_maybe_meta[0])
-            meta = cast(Dict[str, Any], loss_and_maybe_meta[1])
-            loss.backward()
-
-            if meta is not None:
-                with torch.no_grad():
-                    grad_norm = 0.0
-                    for p in list(
-                        filter(lambda p: p.grad is not None, self.parameters())
-                    ):
-                        grad_norm += p.grad.data.norm(2).item() ** 2
-                    meta["grad_norm"] = grad_norm
-            optimizer.step()
-            return loss.item(), meta
-
-        else:
-            warnings.warn(_NO_META_WARNING_MSG)
-            loss_and_maybe_meta.backward()
-            optimizer.step()
-            return loss_and_maybe_meta.item()
+        loss, meta = self.loss(model_in, target)
+        loss.backward()
+        if meta is not None:
+            with torch.no_grad():
+                grad_norm = 0.0
+                for p in list(filter(lambda p: p.grad is not None, self.parameters())):
+                    grad_norm += p.grad.data.norm(2).item() ** 2
+                meta["grad_norm"] = grad_norm
+        optimizer.step()
+        return loss.item(), meta
 
     def reset(
         self, obs: torch.Tensor, rng: Optional[torch.Generator] = None
@@ -251,13 +236,13 @@ class Model(nn.Module, abc.ABC):
     def __len__(self):
         return 1
 
-    def save(self, path: Union[str, pathlib.Path]):
-        """Saves the model to the given path."""
-        torch.save(self.state_dict(), path)
+    def save(self, save_dir: Union[str, pathlib.Path]):
+        """Saves the model to the given directory."""
+        torch.save(self.state_dict(), pathlib.Path(save_dir) / self._MODEL_FNAME)
 
-    def load(self, path: Union[str, pathlib.Path]):
+    def load(self, load_dir: Union[str, pathlib.Path]):
         """Loads the model from the given path."""
-        self.load_state_dict(torch.load(path))
+        self.load_state_dict(torch.load(pathlib.Path(load_dir) / self._MODEL_FNAME))
 
 
 # ---------------------------------------------------------------------------
@@ -311,11 +296,11 @@ class Ensemble(Model, abc.ABC):
         self.deterministic = deterministic
         self.to(device)
 
-    def forward(self, x: ModelInput, **kwargs) -> Tuple[torch.Tensor, ...]:
+    def forward(self, x: torch.Tensor, *args, **kwargs) -> Tuple[torch.Tensor, ...]:
         """Computes the output of the dynamics model.
 
         Args:
-            x (tensor or batch of transitions): the input to the model.
+            x (tensor): the input to the model.
 
         Returns:
             (tuple of tensors): all tensors predicted by the model (e.g., .mean and logvar).
@@ -392,6 +377,7 @@ class Ensemble(Model, abc.ABC):
     def reset(
         self, obs: torch.Tensor, rng: Optional[torch.Generator] = None
     ) -> Dict[str, torch.Tensor]:
+        """Prepares the model for simulating using :class:`mbrl.models.ModelEnv`."""
         raise NotImplementedError(
             "ModelEnv requires 1-D models must be wrapped into a OneDTransitionRewardModel."
         )
