@@ -66,7 +66,9 @@ def gaussian_nll(
 
 # inplace truncated normal function for pytorch.
 # credit to https://github.com/Xingyu-Lin/mbpo_pytorch/blob/main/model.py#L64
-def truncated_normal_(tensor: torch.Tensor, mean: float = 0, std: float = 1):
+def truncated_normal_(
+    tensor: torch.Tensor, mean: float = 0, std: float = 1
+) -> torch.Tensor:
     """Samples from a truncated normal distribution in-place.
 
     Args:
@@ -81,14 +83,11 @@ def truncated_normal_(tensor: torch.Tensor, mean: float = 0, std: float = 1):
     torch.nn.init.normal_(tensor, mean=mean, std=std)
     while True:
         cond = torch.logical_or(tensor < mean - 2 * std, tensor > mean + 2 * std)
-        if not torch.sum(cond):
+        bound_violations = torch.sum(cond).item()
+        if bound_violations == 0:
             break
-        tensor = torch.where(
-            cond,
-            torch.nn.init.normal_(
-                torch.ones(tensor.shape, device=tensor.device), mean=mean, std=std
-            ),
-            tensor,
+        tensor[cond] = torch.normal(
+            mean, std, size=(bound_violations,), device=tensor.device
         )
     return tensor
 
@@ -304,10 +303,10 @@ def propagate(
     raise ValueError(f"Invalid propagation method {propagation_method}.")
 
 
-def rfftfreq(samples: torch.Tensor, device: torch.device) -> torch.Tensor:
+def rfftfreq(samples: int, device: torch.device) -> torch.Tensor:
     if version.parse(torch.__version__) >= version.parse("1.8.0"):
         return torch.fft.rfftfreq(samples, device=device)
-    freqs = np.fft.rfftfreq(samples.cpu().numpy())
+    freqs = np.fft.rfftfreq(samples)  # type: ignore
     return torch.from_numpy(freqs).to(device)
 
 
@@ -365,7 +364,7 @@ def powerlaw_psd_gaussian(
     # Calculate theoretical output standard deviation from scaling
     w = s_scale[1:].detach().clone()
     w[-1] *= (1 + (samples % 2)) / 2.0  # correct f = +-0.5
-    sigma = 2 * torch.sqrt(torch.sum(w ** 2)) / samples
+    sigma = 2 * torch.sqrt(torch.sum(w**2)) / samples
 
     # Adjust size to generate one Fourier component per frequency
     size[-1] = len(f)
@@ -413,7 +412,7 @@ def quantize_obs(
         bit_depth (int): the desired bit depth.
         original_bit_depth (int, optional): the original bit depth, defaults to 8.
         add_noise (bool, optional): if ``True``, uniform noise in the range
-            (0, 1 / 2 ** bit_depth) will be added. Defaults to ``False``.`
+            (0, 2 ** (8 - bit_depth)) will be added. Defaults to ``False``.`
 
     Returns:
         (np.ndarray): the quantized version of the array.
@@ -421,7 +420,7 @@ def quantize_obs(
     ratio = 2 ** (original_bit_depth - bit_depth)
     quantized_obs = (obs // ratio) * ratio
     if add_noise:
-        quantized_obs = quantized_obs.astype(np.double) + np.random.rand(*obs.shape) / (
-            2 ** bit_depth
+        quantized_obs = quantized_obs.astype(np.double) + ratio * np.random.rand(
+            *obs.shape
         )
     return quantized_obs
