@@ -12,6 +12,7 @@ import torch
 import torch.nn as nn
 
 import mbrl.models
+import mbrl.models.util as model_utils
 import mbrl.util.replay_buffer
 from mbrl.env.termination_fns import no_termination
 from mbrl.types import TransitionBatch
@@ -38,20 +39,12 @@ def test_activation_functions_guassian_mlp():
     model2 = mbrl.models.GaussianMLP(
         1, 1, _DEVICE, num_layers=2, activation_fn_cfg=activation_cfg_th
     )
-    model3 = mbrl.models.GaussianMLP(
-        1, 1, _DEVICE, num_layers=2, use_silu=True, activation_fn_cfg=activation_cfg_th
-    )
     hidden_layer = model1.hidden_layers
     silu = torch.nn.SiLU()
     assert str(hidden_layer[0][1]) == str(silu)
     hidden_layer = model2.hidden_layers
     threshold = torch.nn.Threshold(0.5, 10)
     assert str(hidden_layer[0][1]) == str(threshold)
-    hidden_layer = model3.hidden_layers
-    assert str(hidden_layer[0][1]) == str(silu)
-
-
-# use_silu = True ignores activation_cfg for nn.Threshold
 
 
 def test_basic_ensemble_gaussian_forward():
@@ -144,7 +137,7 @@ def test_gaussian_mlp_ensemble_random_model_propagation():
 
     batch_size = 100
     num_reps = 200
-    batch = _OUTPUT_FACTOR * torch.arange(batch_size).view(-1, 1)
+    batch = _OUTPUT_FACTOR * torch.arange(batch_size).view(-1, 1).to(_DEVICE)
     history = ["" for _ in range(batch_size)]
     with torch.no_grad():
         for _ in range(num_reps):
@@ -166,7 +159,7 @@ def test_gaussian_mlp_ensemble_fixed_model_propagation():
 
     batch_size = 100
     num_reps = 200
-    batch = _OUTPUT_FACTOR * torch.arange(batch_size).view(-1, 1)
+    batch = _OUTPUT_FACTOR * torch.arange(batch_size).view(-1, 1).to(_DEVICE)
     history = ["" for _ in range(batch_size)]
     rng = torch.Generator(device=_DEVICE)
     # This creates propagation indices to use for all runs
@@ -351,7 +344,7 @@ class DummyModel(mbrl.models.Model):
         act = x[:, _MOCK_OBS_DIM:]
         new_obs = obs + act.mean(axis=1, keepdim=True)
         # reward is also equal to new_obs
-        return torch.cat([new_obs, new_obs], axis=1)
+        return torch.cat([new_obs, new_obs], dim=1)
 
     def reset_1d(self, _obs, rng=None):
         return {}
@@ -386,7 +379,7 @@ def test_model_env_evaluate_action_sequences():
             expected_returns = horizon * (horizon + 1) * action_sequences[..., 0, 0] / 2
             returns = model_env.evaluate_action_sequences(
                 action_sequences,
-                np.zeros((1, _MOCK_OBS_DIM)),
+                np.zeros(_MOCK_OBS_DIM),
                 num_particles=num_particles,
             )
             assert torch.allclose(expected_returns, returns)
@@ -405,6 +398,7 @@ def test_model_trainer_batch_callback():
             dummy_data,
             dummy_data.squeeze(1),
             dummy_data.squeeze(1),
+            dummy_data.squeeze(1),
         ),
         1,
     )
@@ -428,3 +422,61 @@ def test_model_trainer_batch_callback():
         assert set(counter.keys()) == set(range(num_epochs))
         for i in range(num_epochs):
             assert counter[i] == num_batches
+
+
+def test_conv2d_encoder_shapes():
+    in_channels = 3
+    config = ((in_channels, 32, 3, 2), (32, 64, 3, 2))
+    activation_cls = [nn.ReLU, nn.SiLU, nn.Tanh]
+    encoding_size = 200
+    image_shape = (32, 32)
+    for act_idx, activation_func in enumerate(["ReLU", "SiLU", "Tanh"]):
+        encoder = model_utils.Conv2dEncoder(
+            config, image_shape, encoding_size, activation_func
+        )
+        assert len(encoder.convs) == len(config)
+        for i, layer_cfg in enumerate(config):
+            assert isinstance(encoder.convs[i][0], nn.Conv2d)
+            assert encoder.convs[i][0].in_channels == layer_cfg[0]
+            assert encoder.convs[i][0].out_channels == layer_cfg[1]
+            assert encoder.convs[i][0].kernel_size == (layer_cfg[2], layer_cfg[2])
+            assert encoder.convs[i][0].stride == (layer_cfg[3], layer_cfg[3])
+            assert isinstance(encoder.convs[i][1], activation_cls[act_idx])
+
+        assert isinstance(encoder.fc, nn.Linear)
+        assert encoder.fc.out_features == encoding_size
+
+        dummy = torch.ones((8, in_channels) + image_shape)
+        out = encoder.forward(dummy)
+        assert out.shape == (8, encoding_size)
+
+
+def test_conv2d_decoder_shapes():
+    in_channels = 64
+    config = ((in_channels, 32, 3, 2), (32, 16, 3, 2))
+    activation_cls = [nn.ReLU, nn.SiLU, nn.Tanh]
+    encoding_size = 200
+    deconv_input_shape = (in_channels, 3, 3)
+    for act_idx, activation_func in enumerate(["ReLU", "SiLU", "Tanh"]):
+        decoder = model_utils.Conv2dDecoder(
+            encoding_size, deconv_input_shape, config, activation_func=activation_func
+        )
+        assert len(decoder.deconvs) == len(config)
+        for i, layer_cfg in enumerate(config):
+            if i < len(config) - 1:
+                deconv = decoder.deconvs[i][0]
+                assert isinstance(decoder.deconvs[i][1], activation_cls[act_idx])
+            else:
+                deconv = decoder.deconvs[i]
+            assert isinstance(deconv, nn.ConvTranspose2d)
+            assert deconv.in_channels == layer_cfg[0]
+            assert deconv.out_channels == layer_cfg[1]
+            assert deconv.kernel_size == (layer_cfg[2], layer_cfg[2])
+            assert deconv.stride == (layer_cfg[3], layer_cfg[3])
+
+        assert isinstance(decoder.fc, nn.Linear)
+        assert decoder.fc.out_features == np.prod(deconv_input_shape)
+
+        dummy = torch.ones(8, encoding_size)
+        out = decoder.forward(dummy)
+        assert out.shape[0] == 8 and out.shape[1] == config[-1][1]
